import sys
from argparse import ArgumentParser
from typing import List, Optional

from whippet import whippet
from pathlib import Path


def install_prompt(git_dir: Path) -> bool:
    while True:
        answer = input(
            f"whippet - Are you sure you want to install hooks in {git_dir}? [Y/n] "
        ).lower()
        if answer == "n":
            return False
        if answer in ["y", ""]:
            return True


def make_parser() -> ArgumentParser:
    parser = ArgumentParser(
        prog="whippet", description="Install make based hooks with ease."
    )
    parser.add_argument(
        "-y",
        "--yes",
        "--assume-yes",
        action="store_true",
        help='Automatic yes to prompts; assume "yes" as answer to all prompts and run non-interactively.',
    )
    return parser


def run(cli_args: Optional[List[str]] = None) -> None:
    cwd = Path.cwd()
    git_dir = whippet.resolve_git_dir(cwd)

    if git_dir is None:
        print("whippet - Can not find .git directory, skipping hooks installation.")
        return

    args = make_parser().parse_args(args=cli_args)

    if not (args.yes or install_prompt(git_dir)):
        print("whippet - Aborted.")
        return

    whippet.install_hooks(cwd)


def main() -> None:
    run(sys.argv[1:])



from pathlib import Path
from typing import Optional, List

from whippet import __version__

hook_list: List[str] = [
    "applypatch-msg",
    "pre-applypatch",
    "post-applypatch",
    "pre-commit",
    "prepare-commit-msg",
    "commit-msg",
    "post-commit",
    "pre-rebase",
    "post-checkout",
    "post-merge",
    "pre-push",
    "pre-receive",
    "update",
    "post-receive",
    "post-update",
    "push-to-checkout",
    "pre-auto-gc",
    "post-rewrite",
    "sendemail-validate",
]


def resolve_git_dir(cwd: Path) -> Optional[Path]:
    cwd = cwd.resolve()
    while True:
        git_dir = cwd / ".git"
        if git_dir.exists() and git_dir.is_dir():
            return git_dir
        if cwd == cwd.parent:
            return None
        cwd = cwd.parent


def get_hooks_dir(cwd: Path) -> Optional[Path]:
    git_dir = resolve_git_dir(cwd)
    if git_dir is None:
        return None

    hook_dir = git_dir / "hooks"
    hook_dir.mkdir(mode=0o775, exist_ok=True)
    return hook_dir


def install_hooks(cwd: Path) -> None:
    hooks_dir = get_hooks_dir(cwd)
    if hooks_dir is None:
        print("whippet - Can not find .git directory, skipping hooks installation.")
        return

    template_path = Path(__file__).parent / "hook_template"
    template = template_path.read_text(encoding="utf-8")

    for hook_name in hook_list:
        hook_file = hooks_dir / hook_name

        if hook_file.exists():
            print(f"whippet - {hook_name} hook script already exists - skipping")
            continue

        hook = template.format(version=__version__, hook=hook_name)
        hook_file.write_text(hook, encoding="utf-8")



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['whippet']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['whippet = whippet:console.main']}

setup_kwargs = {
    'name': 'whippet',
    'version': '0.2.2',
    'description': 'Install make based hooks with ease.',
    'long_description': 'Whippet - like husky, but leaner\n################################\nA simple git hooks installer. Inspired by husky_.\n\n.. _husky: https://github.com/typicode/husky#readme\n',
    'author': 'Dalibor Novak',
    'author_email': 'BorePlusPlus@gmail.com',
    'url': 'https://github.com/BorePlusPlus/whippet',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
